@@ -317,11 +317,18 @@ static void noimage(const char *fmt, ...)
 }
 
 /* draws bar indicator when minimum, nominal or maximum values for the given 
-   UPS variable can be determined */
+   UPS variable can be determined.
+   deviation < 0 means that values below nom should be grey instead of
+   green */
 static void drawgeneralbar(double var, int min, int nom, int max, 
 		int deviation, 	const char *format)
 {
-	int	hi, lo, step1, step5, step10;
+	int	hi, lo, step1, step5, step10, graybelownom=0;
+
+	if(deviation < 0) {
+		deviation=-deviation;
+		graybelownom=1;
+	}
 	
 	if ((nom == -1) && ((min == -1) || (max == -1)))
 		noimage("Can't determine range");
@@ -345,19 +352,28 @@ static void drawgeneralbar(double var, int min, int nom, int max,
 		step1 = 1;
 		step5 = 5;
 		step10 = 10;
-	} else {
-		/* the scale is too dense to draw finer scale */
+	} else if((max - min) <= 100) {
 		step1 = 2;
 		step5 = 10;
 		step10 = 20;
+	} else {
+		step1 = 5;
+		step5 = 20;
+		step10 = 40;
 	}
 	
 	/* round min and max points to get high and low numbers for graph */
 	lo = ((min - deviation) / step10) * step10;
 	hi = ((max + deviation + step10/2) / step10) * step10;
 	
-	drawbar(lo, hi, step1, step5, step10, max, hi, lo, min, 
-		nom - deviation, nom + deviation, var, format);
+	if(!graybelownom) {
+		drawbar(lo, hi, step1, step5, step10, max, hi, lo, min, 
+				nom - deviation, nom + deviation, var, format);
+	}
+	else {
+		drawbar(lo, hi, step1, step5, step10, 0, min, max, hi, 
+				nom, max, var, format);
+	}
 
 	/* NOTREACHED */
 }
@@ -367,17 +383,40 @@ static void draw_utility(double var, int min, int nom, int max,
 		int deviation, const char *format)
 {
 	/* hack: deal with hardware that doesn't have known transfer points */
-	if (min == -1) 
-		min = (var < 200.0) ?  90 : 200;
+	if (min == -1) {
+		if(var < 200) {
+			min = 90;
+		}
+		else if(var < 300) {
+			min = 200;
+		}
+		else {
+			min = 340;
+		}
+	}
 
 	/* somewhere between 220 and 230 V, to keep everybody satisfied */
-	if (nom == -1) 
-		nom = (var < 200.0) ? 110 : 225;	
+	if (nom == -1) {
+		if(var < 200) {
+			nom = 110;
+		}
+		else if(var < 300) {
+			nom = 225;
+		}
+		else {
+			nom = 400;
+		}
+	}
 
+	/* symmetrical around nom */
 	if (max == -1) 
-		max = (var < 200.0) ? 140 : 250;
+		max = nom+(nom-min);
+
+	/* Acceptable range of voltage is 85%-110% of nominal voltage
+	 * in EU at least. Be conservative and say +-10% */
+	deviation = nom*0.1;
 	
-	drawgeneralbar(var, min, -1, max, deviation, format);
+	drawgeneralbar(var, min, nom, max, deviation, format);
 
 	/* NOTREACHED */
 }
@@ -393,7 +432,46 @@ static void draw_battpct(double var, int min, int nom, int max,
 static void draw_battvolt(double var, int min, int nom, int max, 
 		int deviation, const char *format)
 {
-	drawbar(10, 40, 1, 5, 10, 0, 20, 28, 40, 25, 27, var, format);
+	if(nom == -1) {
+		/* Use a fixed set of reasonable nominal voltages, seems to
+		 * be the only way to get reasonable behaviour during
+		 * discharge */
+
+		if(var < 9)
+			nom = 6;
+		else if(var < 18)
+			nom = 12;
+		else if(var < 30)
+			nom = 24;
+		else if(var < 60)
+			nom = 48;
+		else if(var < 120)
+			nom = 96;
+		else if(var < 230)
+			nom = 192;
+		else
+			nom = 384;
+
+	}
+
+	if(min == -1) {
+		min = nom/2*1.6+1; /* Assume a 2V cell is dead at 1.6V */
+	}
+
+	if(max == -1) {
+		max = nom/2*2.3+1; /* Assume 2.3V float charge voltage */
+	}
+
+	if (nom < min || nom > max)
+	    	nom = -1;
+
+
+	deviation = -(nom*0.05); /* 5% deviation from nominal voltage */
+	if(deviation==0) {
+		deviation = -1;
+	}
+
+	drawgeneralbar(var, min, nom, max, deviation, format);
 }
 
 /* draws ups.load bar style indicator */
@@ -532,13 +610,38 @@ int main(int argc, char **argv)
 
 struct imgvar_t imgvar[] = {
 	{ "input.voltage", "input.transfer.low", "input.voltage.nominal",
-		"input.transfer.high", 10, 
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L1-N.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L2-N.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L3-N.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L1-L2.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L2-L3.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
+		"%.1f VAC", draw_utility				},
+
+	{ "input.L3-L1.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 0, 
 		"%.1f VAC", draw_utility				},
 
 	{ "battery.charge", NULL, NULL, NULL, 0,
 		"%.1f %%",	draw_battpct				},
 
-	{ "battery.voltage", NULL, NULL, NULL, 0,
+	{ "battery.voltage", "battery.voltage.minimum", "battery.voltage.nominal",
+		"battery.voltage.maximum", 0,
 		"%.1f VDC",	draw_battvolt				},
 
 	{ "ups.temperature", "ups.temperature.minimum", NULL,
@@ -551,8 +654,50 @@ struct imgvar_t imgvar[] = {
 	{ "ups.load", NULL, NULL, NULL, 0,
 		"%.1f %%",	draw_upsload				},
 
+	{ "output.L1.power.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.L2.power.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.L3.power.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.L1.realpower.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.L2.realpower.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.L3.realpower.percent", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
 	{ "output.voltage", "input.transfer.low", "output.voltage.nominal",
-		"input.transfer.high", 10, 
+	       	"input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L1-N.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L2-N.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L3-N.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L1-L2.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L2-L3.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.L3-L1.voltage", "input.transfer.low",
+	       	"output.voltage.nominal", "input.transfer.high", 0, 
 		"%.1f VAC",	draw_utility				},
 
 	{ "output.frequency", NULL, "output.frequency.nominal", NULL, 2,
