@@ -28,6 +28,7 @@
 #include "apc-hid.h"
 #include "extstate.h" /* for ST_FLAG_STRING */
 #include "dstate.h"   /* for STAT_INSTCMD_HANDLED */
+#include "main.h"     /* for getval() */
 #include "common.h"
 
 #define APC_HID_VERSION "APC/CyberPower HID 0.9"
@@ -83,7 +84,7 @@ info_lkp_t apc_date_conversion[] = {
 
 /* APC has two non-NUT-standard status items: "time limit expired" and
    "battery present". The newhidups driver currently ignores
-   batterypresent, and maps timelimitexp to LB. CyberPower has the
+   batterypres, and maps timelimitexp to LB. CyberPower has the
    non-NUT-standard status item "fully charged". The newhidups driver
    currently ignores it. */
 static info_lkp_t timelimitexpired_info[] = {
@@ -220,7 +221,7 @@ static hid_info_t apc_hid2nut[] = {
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.Charging", NULL, "%.0f", HU_FLAG_OK, &charging_info[0] },
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.ShutdownImminent", NULL, "%.0f", HU_FLAG_OK, &shutdownimm_info[0] },
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.BelowRemainingCapacityLimit", NULL, "%.0f", HU_FLAG_OK, &lowbatt_info[0] },
-  { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.OverLoad", NULL, "%.0f", HU_FLAG_OK, &overload_info[0] },
+  { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.Overload", NULL, "%.0f", HU_FLAG_OK, &overload_info[0] },
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.NeedReplacement", NULL, "%.0f", HU_FLAG_OK, &replacebatt_info[0] },
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.RemainingTimeLimitExpired", NULL, "%.0f", HU_FLAG_OK, &timelimitexpired_info[0] },
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.BatteryPresent", NULL, "%.0f", HU_FLAG_OK, &batterypresent_info[0] },
@@ -232,7 +233,7 @@ static hid_info_t apc_hid2nut[] = {
   { "ups.status", 0, 1, "UPS.PowerSummary.ShutdownImminent", NULL, "%.0f", HU_FLAG_OK, &shutdownimm_info[0] },
 
   { "ups.status", 0, 1, "UPS.PowerSummary.PresentStatus.FullyCharged", NULL, "%.0f", HU_FLAG_OK, &fullycharged_info[0] }, /* CyberPower */
-  { "ups.status", 0, 1, "UPS.Output.OverLoad", NULL, "%.0f", HU_FLAG_OK, &overload_info[0] }, /* CyberPower */
+  { "ups.status", 0, 1, "UPS.Output.Overload", NULL, "%.0f", HU_FLAG_OK, &overload_info[0] }, /* CyberPower */
   { "ups.status", 0, 1, "UPS.Output.Boost", NULL, "%.0f", HU_FLAG_OK, &boost_info[0] }, /* CyberPower */
 
   /* Input page */
@@ -243,7 +244,7 @@ static hid_info_t apc_hid2nut[] = {
 
   /* Output page */
   { "output.voltage", 0, 0, "UPS.Output.Voltage", NULL, "%.1f", HU_FLAG_OK, NULL },
-  { "output.voltage.target.line", 0, 0, "UPS.Output.ConfigVoltage", NULL, "%.1f", HU_FLAG_OK, NULL },
+  { "output.voltage.nominal", 0, 0, "UPS.Output.ConfigVoltage", NULL, "%.1f", HU_FLAG_OK, NULL },
 
   /* instant commands. */
   /* test.* split into subset while waiting for extradata support
@@ -351,9 +352,38 @@ static char *apc_format_serial(HIDDevice *hd) {
  * the device is supported by this subdriver, else 0. */
 static int apc_claim(HIDDevice *hd) {
 	if (hd->VendorID == APC_VENDORID) {
-		return 1;
+		switch (hd->ProductID) {
+		case  0x0002:
+			return 1;  /* accept known UPSs */
+		default:
+			if (getval("productid")) {
+				return 1;
+			} else {
+			upsdebugx(1,
+"This particular APC device (%04x/%04x) is not (or perhaps not yet)\n"
+"supported by newhidups. Try running the driver with the '-x productid=%04x'\n"
+"option. Please report your results to the NUT developer's mailing list.\n",
+						 hd->VendorID, hd->ProductID, hd->ProductID);
+			return 0;
+			}
+		}
 	} else if (hd->VendorID == CPS_VENDORID) {
-		return 1;
+		switch (hd->ProductID) {
+		case 0x0005:
+		case 0x0501:
+			return 1;  /* accept known UPSs */
+		default:
+			if (getval("productid")) {
+				return 1;
+			} else {
+			upsdebugx(1,
+"This particular CyberPower device (%04x/%04x) is not (or perhaps not yet)\n"
+"supported by newhidups. Try running the driver with the '-x productid=%04x'\n"
+"option. Please report your results to the NUT developer's mailing list.\n",
+						 hd->VendorID, hd->ProductID, hd->ProductID);
+			return 0;
+			}
+		}
 	} else {
 		return 0;
 	}
