@@ -20,7 +20,7 @@
 
 #include <ctype.h>
 #include <sys/ioctl.h>
-#include <sys/termios.h>
+#include "serial.h"
 #include "timehead.h"
 
 #define APC_TABLE_VERSION	"version 2.0"
@@ -152,7 +152,7 @@ struct apc_vartab_t {
 
 	{ "output.current", 	APC_POLL|APC_F_AMP,	'/' },
 	{ "output.voltage", 	APC_POLL|APC_F_VOLT,	'O' },
-	{ "output.voltage.target.battery",  
+	{ "output.voltage.nominal",  
 				APC_F_VOLT,		'o' },
 
 	{ "ambient.humidity",  	APC_POLL|APC_F_PERCENT,	'h' },
@@ -256,6 +256,10 @@ struct {
 	{ "6TI",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	/* CS 350 */
 	{ "5.4.D",	"\1ABPQRSUYbdfgjmnx9",	0 },
+	/* Smart-UPS 600 */
+	{ "D6",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D5",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
+	{ "D4",         "789ABCEFGKLMNOPQRSUVWXYZ", 0 },
 
 	{ NULL,		NULL,			0 },
 };
