@@ -41,10 +41,26 @@
 /* Config block offsets */
 #define BCMXCP_CONFIG_BLOCK_NOMINAL_OUTPUT_VOLTAGE 	8
 #define BCMXCP_CONFIG_BLOCK_HW_MODULES_INSTALLED_BYTE4 	16
+#define BCMXCP_CONFIG_BLOCK_EXTENDED_BLOCK_LENGTH	47
 #define BCMXCP_CONFIG_BLOCK_SERIAL_NUMBER		64
 
-/* Extended	limits block offsets */
+/* Index for Extende Limits block offsets */
+#define BCMXCP_EXT_LIMITS_BLOCK_NOMINAL_INPUT_VOLTAGE	0
+#define BCMXCP_EXT_LIMITS_BLOCK_NOMINAL_INPUT_FREQ	2
+#define BCMXCP_EXT_LIMITS_BLOCK_NOMINAL_TRUE_POWER	4
+#define BCMXCP_EXT_LIMITS_BLOCK_COMM_SPEC_VERSION	6
+#define BCMXCP_EXT_LIMITS_BLOCK_FREQ_DEV_LIMIT		8
+#define BCMXCP_EXT_LIMITS_BLOCK_VOLTAGE_LOW_DEV_LIMIT	10
+#define BCMXCP_EXT_LIMITS_BLOCK_VOLTAGE_HIGE_DEV_LIMIT	12
+#define BCMXCP_EXT_LIMITS_BLOCK_PHASE_DEV_LIMIT		14
 #define BCMXCP_EXT_LIMITS_BLOCK_LOW_BATT_WARNING	16
+#define BCMXCP_EXT_LIMITS_BLOCK_HORN_STATUS		17
+#define BCMXCP_EXT_LIMITS_BLOCK_MIN_INPUT_VOLTAGE	18
+#define BCMXCP_EXT_LIMITS_BLOCK_MAX_INPUT_VOLTAGE	20
+#define BCMXCP_EXT_LIMITS_BLOCK_RETURN_STAB_DELAY	22
+#define BCMXCP_EXT_LIMITS_BLOCK_BATT_CAPACITY_RETURN	24
+#define BCMXCP_EXT_LIMITS_BLOCK_AMBIENT_TEMP_LOW	25
+#define BCMXCP_EXT_LIMITS_BLOCK_AMBIENT_TEMP_HIGE	26
 
 /* Meter map offsets used	*/
 #define BCMXCP_METER_MAP_OUTPUT_VA			23
