@@ -57,7 +57,7 @@ static void sock_fail(const char *fn)
 	user = getpwuid(getuid());
 
 	if (!user)
-		fatal("getpwuid");
+		fatal_with_errno("getpwuid");
 
 	/* deal with some common problems */
 	switch (errno) {
@@ -101,7 +101,7 @@ static int open_sock(const char *fn)
 	fd = socket(AF_UNIX, SOCK_STREAM, 0);
 
 	if (fd < 0)
-		fatal("Can't create a unix domain socket");
+		fatal_with_errno("Can't create a unix domain socket");
 
 	/* keep this around for the unlink() when exiting */
 	sockfn = xstrdup(fn);
@@ -122,12 +122,12 @@ static int open_sock(const char *fn)
 	ret = chmod(sockfn, 0660);
 
 	if (ret < 0)
-		fatal("chmod(%s, 0660) failed", sockfn);
+		fatal_with_errno("chmod(%s, 0660) failed", sockfn);
 
 	ret = listen(fd, DS_LISTEN_BACKLOG);
 
 	if (ret < 0)
-		fatal("listen(%d, %d) failed", fd, DS_LISTEN_BACKLOG);
+		fatal_with_errno("listen(%d, %d) failed", fd, DS_LISTEN_BACKLOG);
 
 	return fd;
 }
@@ -224,7 +224,7 @@ static void conn_add(int fd)
 	acc = accept(fd, (struct sockaddr *) &sa, &salen);
 
 	if (acc < 0) {
-		upslog(LOG_ERR, "accept on unix fd failed");
+		upslog_with_errno(LOG_ERR, "accept on unix fd failed");
 		return;
 	}
 
@@ -233,7 +233,7 @@ static void conn_add(int fd)
 	ret = fcntl(acc, F_GETFL, 0);
 
 	if (ret < 0) {
-		upslog(LOG_ERR, "fcntl get on unix fd failed");
+		upslog_with_errno(LOG_ERR, "fcntl get on unix fd failed");
 		close(acc);
 		return;
 	}
@@ -241,7 +241,7 @@ static void conn_add(int fd)
 	ret = fcntl(acc, F_SETFL, ret | O_NDELAY);
 
 	if (ret < 0) {
-		upslog(LOG_ERR, "fcntl set O_NDELAY on unix fd failed");
+		upslog_with_errno(LOG_ERR, "fcntl set O_NDELAY on unix fd failed");
 		close(acc);
 		return;
 	}	
@@ -473,8 +473,13 @@ void dstate_init(const char *prog, const char *port)
 	/* do this here for now */
 	signal(SIGPIPE, SIG_IGN);
 
-	snprintf(sockname, sizeof(sockname), "%s/%s-%s", 
-		dflt_statepath(), prog, port);
+	if (port != NULL)
+		snprintf(sockname, sizeof(sockname), "%s/%s-%s",
+			dflt_statepath(), prog, port);
+	else
+		snprintf(sockname, sizeof(sockname), "%s/%s",
+			dflt_statepath(), prog);
+
 	sockfd = open_sock(sockname);
 
 	upsdebugx(2, "dstate_init: sock %s open on fd %d", sockname, sockfd);
@@ -518,7 +523,7 @@ int dstate_poll_fds(int interval, int extrafd)
 	if (ret < 0) {
 		/* ignore interruptions from signals */
 		if (errno != EINTR)
-			upslog(LOG_ERR, "select unix sockets failed");
+			upslog_with_errno(LOG_ERR, "select unix sockets failed");
 		return 0;
 	}
 
@@ -638,45 +643,18 @@ void dstate_setaux(const char *var, int aux)
 
 const char *dstate_getinfo(const char *var)
 {
-	struct	st_tree_t	*sttmp;
-
-	/* find the dtree node for var */
-	sttmp = state_tree_find(dtree_root, var);
-
-	if (!sttmp)
-		return NULL;
-
-	return sttmp->val;
+	return state_getinfo(dtree_root, var);
 }
 
 void dstate_addcmd(const char *cmdname)
 {
-	struct cmdlist_t *tmp, *last;
+	int	ret;
 
-	tmp = cmdhead;
-	last = NULL;
-
-	while (tmp) {
-		last = tmp;
-
-		/* no duplicates allowed */
-		if (!strcmp(tmp->name, cmdname))
-			return;
-
-		tmp = tmp->next;
-	}
-
-	tmp = xmalloc(sizeof(struct cmdlist_t));
-	tmp->name = xstrdup(cmdname);
-	tmp->next = NULL;
-
-	if (last)
-		last->next = tmp;
-	else
-		cmdhead = tmp;
+	ret = state_addcmd(&cmdhead, cmdname);
 
 	/* update listeners */
-	send_to_all("ADDCMD %s\n", cmdname);
+	if (ret == 1)
+		send_to_all("ADDCMD %s\n", cmdname);
 }
 
 int dstate_delinfo(const char *var)
@@ -694,77 +672,32 @@ int dstate_delinfo(const char *var)
 
 int dstate_delenum(const char *var, const char *val)
 {
-	struct	st_tree_t	*sttmp;
-	struct	enum_t *etmp, *elast;
+	int	ret;
 
-	/* find the dtree node for var */
-	sttmp = state_tree_find(dtree_root, var);
+	ret = state_delenum(dtree_root, var, val);
 
-	if (!sttmp)
-		return 0;
+	/* update listeners */
+	if (ret == 1)
+		send_to_all("DELENUM %s \"%s\"\n", var, val);
 
-	/* look for val in enum_list */
-	etmp = sttmp->enum_list;
-	elast = NULL;	
-
-	while (etmp) {
-		if (!strcmp(etmp->val, val)) {
-
-			if (elast)
-				elast->next = etmp->next;
-			else
-				sttmp->enum_list = etmp->next;
-
-			/* update listeners */
-			send_to_all("DELENUM %s \"%s\"\n", var, etmp->val);
-
-			free(etmp->val);
-			free(etmp);
-
-			return 1;
-		}
-
-		elast = etmp;
-		etmp = etmp->next;
-	}
-
-	return 0;	/* not found */
+	return ret;
 }
 
 int dstate_delcmd(const char *cmd)
 {
-	struct	cmdlist_t	*ctmp, *clast;
+	int	ret;
 
-	ctmp = cmdhead;
-	clast = NULL;	
+	ret = state_delcmd(&cmdhead, cmd);
 
-	while (ctmp) {
-		if (!strcmp(ctmp->name, cmd)) {
+	/* update listeners */
+	if (ret == 1)
+		send_to_all("DELCMD %s\n", cmd);
 
-			if (clast)
-				clast->next = ctmp->next;
-			else
-				cmdhead = ctmp->next;
-
-			/* update listeners */
-			send_to_all("DELCMD %s\n", cmd);
-
-			free(ctmp->name);
-			free(ctmp);
-
-			return 1;
-		}
-
-		ctmp = ctmp->next;
-	}
-
-	return 0;	/* not found */
+	return ret;
 }
 
 void dstate_free(void)
 {
-	struct	cmdlist_t	*ctmp, *cnext;
-
 	if (sockfd != -1) {
 		close(sockfd);
 		sockfd = -1;
@@ -779,18 +712,7 @@ void dstate_free(void)
 	state_infofree(dtree_root);
 	dtree_root = NULL;
 	
-	ctmp = cmdhead;
-
-	while (ctmp) {
-		cnext = ctmp->next;
-
-		if (ctmp->name)
-			free(ctmp->name);
-		free(ctmp);
-
-		ctmp = cnext;
-	}
-
+	state_cmdfree(cmdhead);
 	cmdhead = NULL;
 
 	conn_close_all();
