@@ -20,6 +20,8 @@
 #ifndef DSTATE_H_SEEN
 #define DSTATE_H_SEEN 1
 
+#include "attribute.h"
+
 #include "parseconf.h"
 #include "upshandler.h"
 
@@ -47,8 +49,10 @@ struct conn_t {
 
 void dstate_init(const char *prog, const char *port);
 int dstate_poll_fds(int interval, int extrafd);
-int dstate_setinfo(const char *var, const char *fmt, ...);
-int dstate_addenum(const char *var, const char *fmt, ...);
+int dstate_setinfo(const char *var, const char *fmt, ...)
+	__attribute__ ((__format__ (__printf__, 2, 3)));
+int dstate_addenum(const char *var, const char *fmt, ...)
+	__attribute__ ((__format__ (__printf__, 2, 3)));
 void dstate_setflags(const char *var, int flags);
 void dstate_setaux(const char *var, int aux);
 const char *dstate_getinfo(const char *var);
