@@ -4,7 +4,6 @@
  *
  * @author Copyright (C) 2003 - 2005
  *	Arnaud Quette <arnaud.quette@free.fr> && <arnaud.quette@mgeups.com>
- *	Philippe Marzouk <philm@users.sourceforge.net> (dump_hex())
  *	John Stamp <kinsayder@hotmail.com>
  *      2005 Peter Selinger <selinger@users.sourceforge.net>
  *	
@@ -36,29 +35,199 @@
 #include "hidparser.h"
 #include "hidtypes.h"
 #include "libhid.h"
+#include "common.h" /* for xmalloc, upsdebugx prototypes */
 
-#include "libusb.h"
+/* Communication layers and drivers (USB and MGE SHUT) */
+#ifdef SHUT_MODE
+	#include "libshut.h"
+	communication_subdriver_t *comm_driver = &shut_subdriver;
+#else
+	#include "libusb.h"
+	communication_subdriver_t *comm_driver = &usb_subdriver;
+#endif
 
 #include <errno.h>
 
-/* structure to describe a retrieved report */
-struct report_s {
-	int id;                  /* report ID */
-	time_t ts;               /* timestamp when the report was retrieved */
-	int len;                 /* size of report data */
-	unsigned char data[100]; /* report data */
+/* support functions */
+static float logical_to_physical(HIDData *Data, long logical);
+static long physical_to_logical(HIDData *Data, float physical);
+static const char *hid_lookup_path(unsigned int usage, usage_tables_t *utab);
+static int hid_lookup_usage(char *name, usage_tables_t *utab);
+static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab);
+static int path_to_string(char *HIDpath, HIDPath *path, usage_tables_t *utab);
+static long get_unit_expo(long UnitType);
+static float expo(int a, int b);
+
+/* report buffer structure: holds data about most recent report for
+   each given report id */
+struct reportbuf_s {
+       time_t ts[256];           /* timestamp when report was retrieved */
+       int len[256];             /* size of report data */
+       unsigned char *data[256]; /* report data (allocated) */
 };
-typedef struct report_s report_t;
+typedef struct reportbuf_s reportbuf_t;
 
 /* global variables */
 
-static HIDData   	hData;
-static HIDParser 	hParser;
-/* the most recently retrieved report */
-static report_t cur_report_struct = {0, 0, 0, {0}};
+static HIDDesc         *pDesc = NULL; /* parsed Report Descriptor */
+static reportbuf_t      *rbuf = NULL;  /* buffer for most recent reports */
+
+#define min(x,y) ((x)>(y) ? (y) : (x))
+
+/* ---------------------------------------------------------------------- */
+/* report buffering system */
+
+/* HID data items are retrieved via "reports". Each report is
+   identified by a report ID, which is an integer in the range
+   0-255. Each report can hold several items. To avoid retrieving a
+   given report multiple times in short succession, we use a data
+   structure called a "report buffer". The functions in this group
+   operate on entire *reports*, not individual data items. */
+
+/* allocate a new report buffer. Return pointer on success, else NULL
+   with errno set. The returned data structure must later be freed
+   with free_report_buffer(). */
+reportbuf_t *new_report_buffer(void) {
+	reportbuf_t *rbuf;
+	int i;
+
+	rbuf = malloc(sizeof(reportbuf_t));
+	if (!rbuf) {
+		return NULL;
+	}
+	for (i=0; i<256; i++) {
+		rbuf->ts[i] = 0;
+		rbuf->len[i] = 0;
+		rbuf->data[i] = NULL;
+	}
+	return rbuf;
+}
+
+void free_report_buffer(reportbuf_t *rbuf) {
+	int i;
+
+	if (rbuf) {
+		for (i=0; i<256; i++) {
+			free(rbuf->data[i]);
+		}
+	}
+	free(rbuf);
+}
+
+/* refresh the report with the given id in the report buffer rbuf.  If
+   the report is not yet in the buffer, or if it is older than "age"
+   seconds, then the report is freshly read from the USB
+   device. Otherwise, it is unchanged. Return 0 on success, -1 on
+   error with errno set. */
+int refresh_report_buffer(reportbuf_t *rbuf, int id, int age, HIDDesc *pDesc, hid_dev_handle *udev) {
+	int len = pDesc->replen[id]; /* length of report */
+	unsigned char *data;
+	int r;
+
+	if (rbuf->data[id] != NULL && rbuf->ts[id] + age > time(0)) {
+		/* buffered report is still good; nothing to do */
+		return 0;
+	}
+
+	data = malloc(len+1); /* first byte holds report id */
+	if (!data) {
+		return -1;
+	}
+	memset(data, 0, len+1);
+	r = comm_driver->get_report(udev, id, data, len+1);
+	if (r <= 0) {
+		return -1;
+	}
+	/* have valid report */
+	free(rbuf->data[id]);
+	rbuf->data[id] = data;
+	rbuf->ts[id] = time(0);
+	rbuf->len[id] = r;  /* normally equal to len+1, but could be less? */
+
+	upsdebug_hex (3, "Report[r]", rbuf->data[id], rbuf->len[id]);
+	
+	return 0;
+}
+
+/* send the report with the given id to the HID device. Return 0 on
+ * success, or -1 on failure with errno set. */
+int set_report_from_buffer(reportbuf_t *rbuf, int id, hid_dev_handle *udev) {
+	int r;
+
+	r = comm_driver->set_report(udev, id, rbuf->data[id], rbuf->len[id]);
+	if (r <= 0) {
+		return -1;
+	}
+	return 0;
+}	
+
+/* file a given report in the report buffer. This is used when the
+   report has been obtained without having been explicitly requested,
+   e.g., it arrived through an interrupt transfer. Returns 0 on
+   success, -1 on error with errno set. Note: len >= 1, and the first
+   byte holds the report id. */
+int file_report_buffer(reportbuf_t *rbuf, u_char *report, int len) {
+	unsigned char *data;
+	int id;
+
+	id = report[0];
+	data = malloc(len);
+	if (!data) {
+		return -1;
+	}
+	memcpy(data, report, len);
+
+	/* have valid report */
+	free(rbuf->data[id]);
+	rbuf->data[id] = data;
+	rbuf->ts[id] = time(0);
+	rbuf->len[id] = len;
+
+	upsdebug_hex (3, "Report[i]", rbuf->data[id], rbuf->len[id]);
+	
+	return 0;
+}
+
+/* ---------------------------------------------------------------------- */
+/* the functions in this next group operate on buffered reports, but
+   operate on individual items, not whole reports. */
+
+/* read the logical value for the given pData. No logical to physical
+   conversion is performed. If age>0, the read operation is buffered
+   if the item's age is less than "age". On success, return 0 and
+   store the answer in *value. On failure, return -1 and set errno. */
+int get_item_buffered(reportbuf_t *rbuf, HIDData *pData, int age, HIDDesc *pDesc, hid_dev_handle *udev, long *Value) {
+	int r;
+	int id;
+
+	id = pData->ReportID;
+
+	r = refresh_report_buffer(rbuf, id, age, pDesc, udev);
+	if (r<0) {
+		return -1;
+	}
+	GetValue(rbuf->data[id], pData, Value);
+	return 0;
+}
+
+/* set the logical value for the given pData. No physical to logical
+   conversion is performed. On success, return 0, and failure, return
+   -1 and set errno. The updated value is sent to the device, and also
+   stored in the local buffer. */
+int set_item_buffered(reportbuf_t *rbuf, HIDData *pData, HIDDesc *pDesc, hid_dev_handle *udev, long Value) {
+	int id, r;
+
+	id = pData->ReportID;
+
+	SetValue(pData, rbuf->data[id], Value);
+	r = set_report_from_buffer(rbuf, id, udev);
+	return r;
+}
+
+/* ---------------------------------------------------------------------- */
 
 /* FIXME: we currently "hard-wire" the report buffer size in the calls
-   to libusb_get_report() below to 8 bytes. This is not really a great
+   to libxxx_get_report() below to 8 bytes. This is not really a great
    idea, but it is necessary because Belkin models will crash,
    sometimes with permanent firmware damage, if called with a larger
    buffer size (never mind the USB specification). Let's hope for now
@@ -67,12 +236,6 @@ static report_t cur_report_struct = {0, 0, 0, {0}};
    report buffer from the report descriptor. */
 #define REPORT_SIZE 8
 
-/* TODO: rework all that */
-void upsdebugx(int level, const char *fmt, ...);
-#define TRACE upsdebugx
-
-#define min(x,y) ((x)>(y) ? (y) : (x))
-
 /* Units and exponents table (HID PDC, 3.2.3) */
 #define NB_HID_UNITS 10
 static const long HIDUnits[NB_HID_UNITS][2]=
@@ -89,17 +252,6 @@ static const long HIDUnits[NB_HID_UNITS][2]=
 	{0x00101001,0}, /* As */
 };
 
-/* support functions */
-static float logical_to_physical(HIDData *Data, long logical);
-static long physical_to_logical(HIDData *Data, float physical);
-static const char *hid_lookup_path(unsigned int usage, usage_tables_t *utab);
-static int hid_lookup_usage(char *name, usage_tables_t *utab);
-static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab);
-static int path_to_string(char *HIDpath, HIDPath *path, usage_tables_t *utab);
-static void dump_hex (const char *msg, const unsigned char *buf, int len);
-static long get_unit_expo(long UnitType);
-static float expo(int a, int b);
-
 /* ---------------------------------------------------------------------- */
 /* matchers */
 
@@ -372,29 +524,23 @@ void free_regex_matcher(HIDDeviceMatcher_t *matcher) {
 
 /* ---------------------------------------------------------------------- */
 
-
-void HIDDumpTree(usb_dev_handle *udev, usage_tables_t *utab)
+void HIDDumpTree(hid_dev_handle *udev, usage_tables_t *utab)
 {
-	int 		i;
+	int 		j;
 	char 		path[128], type[10];
 	float		value;
-	HIDData 	tmpData;
-	HIDParser 	tmpParser;
+	HIDData 	*pData;
 
-	while (HIDParse(&hParser, &tmpData) != FALSE)
+	for (j=0; j<pDesc->nitems; j++)
 	{
+		pData = &pDesc->item[j];
+
 		/* Build the path */
-		path[0] = '\0';
-		for (i = 0; i < tmpData.Path.Size; i++)
-		{
-			strcat(path, hid_lookup_path((tmpData.Path.Node[i].UPage * 0x10000) + tmpData.Path.Node[i].Usage, utab));
-			if (i < (tmpData.Path.Size - 1))
-				strcat (path, ".");
-		}
+		path_to_string(path, &pData->Path, utab);
 
 		/* Get data type */
 		type[0] = '\0';
-		switch (tmpData.Type)
+		switch (pData->Type)
 		{
 			case ITEM_FEATURE:
 				strcat(type, "Feature");
@@ -412,18 +558,13 @@ void HIDDumpTree(usb_dev_handle *udev, usage_tables_t *utab)
 
 		/* FIXME: enhance this or fix/change the HID parser (see libhid project) */
 		if ( strstr(path, "000000") == NULL) {
-			/* Backup shared data */
-			memcpy(&tmpParser, &hParser, sizeof (hParser));
 
 			/* Get data value */
 			if (HIDGetItemValue(udev, path, &value, utab) > 0)
-				TRACE(1, "Path: %s, Type: %s, Value: %f", path, type, value);
+				upsdebugx(1, "Path: %s, Type: %s, Value: %f", path, type, value);
 			
 			else
-				TRACE(1, "Path: %s, Type: %s", path, type);
-
-			/* Restore shared data */
-			memcpy(&hParser, &tmpParser, sizeof (tmpParser));
+				upsdebugx(1, "Path: %s, Type: %s", path, type);
 		}
 	}
 }
@@ -431,18 +572,18 @@ void HIDDumpTree(usb_dev_handle *udev, usage_tables_t *utab)
 /* Matcher is a linked list of matchers (see libhid.h), and the opened
     device must match all of them. On success, set *udevp and *hd and
     return hd. On failure, return NULL. */
-HIDDevice *HIDOpenDevice(usb_dev_handle **udevp, HIDDevice *hd, HIDDeviceMatcher_t *matcher, int mode)
+HIDDevice *HIDOpenDevice(hid_dev_handle **udevp, HIDDevice *hd, HIDDeviceMatcher_t *matcher, int mode)
 {
 	int ReportSize;
 	unsigned char ReportDesc[4096];
 
 	if ( mode == MODE_REOPEN )
 	{
-		TRACE(2, "Reopening device");
+		upsdebugx(2, "Reopening device");
 	}
 
 	/* get and parse descriptors (dev, cfg and report) */
-	ReportSize = libusb_open(udevp, hd, matcher, ReportDesc, mode);
+	ReportSize = comm_driver->open(udevp, hd, matcher, ReportDesc, mode);
 
 	if (ReportSize == -1)
 		return NULL;
@@ -450,89 +591,105 @@ HIDDevice *HIDOpenDevice(usb_dev_handle **udevp, HIDDevice *hd, HIDDeviceMatcher
 	{
 		if ( mode == MODE_REOPEN )
 		{
-			TRACE(2, "Device reopened successfully");
+			upsdebugx(2, "Device reopened successfully");
 			return hd;
 		}
 	
-		TRACE(2, "Report Descriptor size = %d", ReportSize);
-		dump_hex ("Report Descriptor", ReportDesc, 200);
+		upsdebugx(2, "Report Descriptor size = %d", ReportSize);
+		upsdebug_hex(3, "Report Descriptor", ReportDesc, ReportSize);
 
-		/* HID Parser Init */
-		ResetParser(&hParser);
-		hParser.ReportDescSize = ReportSize;
-		memcpy(hParser.ReportDesc, ReportDesc, ReportSize);
+		/* Parse Report Descriptor */
+		Free_ReportDesc(pDesc);
+		pDesc = Parse_ReportDesc(ReportDesc, ReportSize);
+		if (!pDesc) {
+			upsdebugx(0, "Failed to parse report descriptor: %s", strerror(errno));
+			HIDCloseDevice(*udevp);
+			*udevp = NULL;
+			return NULL;
+		}
 
+		/* prepare report buffer */
+		free_report_buffer(rbuf);
+		rbuf = new_report_buffer();
+		if (!rbuf) {
+			upsdebugx(0, "Failed to allocate report buffer: %s", strerror(errno));
+			HIDCloseDevice(*udevp);
+			*udevp = NULL;
+			return NULL;
+		}
 	}
 	return hd;
 }
 
-/* int HIDGetItem(hid_info_t *ItemInfo, HIDItem *item) */
-HIDItem *HIDGetItem(const char *ItemPath)
+/* return 1 if OK, 0 on fail, <= -1 otherwise (ie disconnect). TODO:
+   return value should be checked. Returns the logical value
+   associated with the given path in *Value (i.e., don't do any
+   logical->physical conversion. Also returns pointer to the
+   corresponding HIDData item in *ppData, if ppData!=NULL. */
+static int HIDGetItemLogical(hid_dev_handle *udev, char *path, usage_tables_t *utab, long *Value, HIDData **ppData)
 {
-	/* was for libhid, not useful in our scope! */
-	return NULL;
-}
+	int i, r;
+	long hValue;
+	HIDData *pData;
+	HIDPath Path;
 
-/* return 1 if OK, 0 on fail, <= -1 otherwise (ie disconnect) */
-int HIDGetItemValue(usb_dev_handle *udev, char *path, float *Value, usage_tables_t *utab)
-{
-	int i, retcode;
-	float physical;
-	report_t *cur_report = &cur_report_struct;
-
-	/* Prepare path of HID object */
-	hData.Type = ITEM_FEATURE;
-	hData.ReportID = 0;
-
-	if((retcode = string_to_path(path, &hData.Path, utab)) <= 0) {
-		return 0; /* TODO: should be checked */
+	r = string_to_path(path, &Path, utab);
+	if (r <= 0) {
+		return 0;
 	}
 
-	TRACE(4, "Path depth = %i", hData.Path.Size);
+	upsdebugx(4, "Path depth = %i", Path.Size);
 	
-	for (i = 0; i<hData.Path.Size; i++) {
-		TRACE(4, "%i: UPage(%x), Usage(%x)", i, hData.Path.Node[i].UPage, hData.Path.Node[i].Usage);
+	for (i = 0; i<Path.Size; i++) {
+		upsdebugx(4, "%i: Usage(%08x)", i, Path.Node[i]);
 	} 
 
 	/* Get info on object (reportID, offset and size) */
-	if (FindObject(&hParser, &hData) != 1) {
-		TRACE(2, "Can't find object %s", path);
-		return 0; /* TODO: should be checked */
+	pData = FindObject_with_Path(pDesc, &Path, ITEM_FEATURE);
+	if (!pData) {
+		upsdebugx(2, "Can't find object %s", path);
+		return 0;
 	} 
-	/* Get report with data */
-	/* Bufferize at least the last report */
-	if (cur_report->id != hData.ReportID || time(NULL) > cur_report->ts + MAX_TS) {
-		/* report is not in buffer or too old;
-		   need to retrieve report */
-		retcode = libusb_get_report(udev, hData.ReportID, cur_report->data, REPORT_SIZE);
-		if (retcode <= 0) {
-			TRACE(2, "Can't retrieve Report %i (%i/%i): %s", hData.ReportID, retcode, errno, strerror(errno));
-			return -errno;
-		} else {
-			cur_report->len = retcode;
-			cur_report->ts = time(NULL);
-		}
+	r = get_item_buffered(rbuf, pData, MAX_TS, pDesc, udev, &hValue);
+	if (r<0) {
+		upsdebugx(2, "Can't retrieve Report %i (%i): %s", pData->ReportID, errno, strerror(errno));
+		return -errno;
+	}
+
+	*Value = hValue;
+	if (ppData != NULL) {
+		*ppData = pData;
+	}
+	return 1;
+}
+
+/* return 1 if OK, 0 on fail, -errno otherwise (ie disconnect). TODO:
+   return value should be checked. Return the physical value
+   associated with the given path. */
+int HIDGetItemValue(hid_dev_handle *udev, char *path, float *Value, usage_tables_t *utab)
+{
+	int r;
+	float physical;
+	long hValue;
+	HIDData *pData;
+
+	r = HIDGetItemLogical(udev, path, utab, &hValue, &pData);
+	if (r <= 0) {
+		return r;
 	}
-	/* have valid report now */
 	
-	/* Extract the data value */
-	GetValue((const unsigned char *) cur_report->data, &hData);
-	cur_report->id = hData.ReportID;
-	
-	TRACE(4, "=>> Before exponent: %ld, %i/%i)", hData.Value,
-	      (int)hData.UnitExp, (int)get_unit_expo(hData.Unit) );
+	upsdebugx(4, "=>> Before exponent: %ld, %i/%i)", hValue,
+	      (int)pData->UnitExp, (int)get_unit_expo(pData->Unit) );
 	
 	/* Convert Logical Min, Max and Value into Physical */
-	physical = logical_to_physical(&hData, hData.Value);
+	physical = logical_to_physical(pData, hValue);
 	
 	/* Process exponents and units */
-	physical *= (float) expo(10,(int)hData.UnitExp - get_unit_expo(hData.Unit));
-	hData.Value = (long) physical;
-	
-	TRACE(4, "=>> After conversion: %f (%ld), %i/%i)", physical, 
-	      hData.Value, (int)hData.UnitExp, (int)get_unit_expo(hData.Unit));
-	
-	dump_hex ("Report ", cur_report->data, cur_report->len);
+	physical *= (float) expo(10,(int)pData->UnitExp - get_unit_expo(pData->Unit));
+	hValue = (long) physical;
+
+	upsdebugx(4, "=>> After conversion: %f (%ld), %i/%i)", physical,
+	      hValue, (int)pData->UnitExp, (int)get_unit_expo(pData->Unit));
 	
 	*Value = physical;
 	return 1;
@@ -540,164 +697,168 @@ int HIDGetItemValue(usb_dev_handle *udev, char *path, float *Value, usage_tables
 
 /* rawbuf must point to a large enough buffer to hold the resulting
  * string. Return pointer to rawbuf on success, NULL on failure. */
-char *HIDGetItemString(usb_dev_handle *udev, char *path, unsigned char *rawbuf, usage_tables_t *utab)
+char *HIDGetItemString(hid_dev_handle *udev, char *path, unsigned char *rawbuf, usage_tables_t *utab)
 {
-  int i, retcode;
-  
-  /* Prepare path of HID object */
-  hData.Type = ITEM_FEATURE;
-  hData.ReportID = 0;
-  
-  if((retcode = string_to_path(path, &hData.Path, utab)) > 0) {
-    TRACE(4, "Path depth = %i", retcode);
-    
-    for (i = 0; i<retcode; i++)
-      TRACE(4, "%i: UPage(%x), Usage(%x)", i,
-		hData.Path.Node[i].UPage,
-		hData.Path.Node[i].Usage);
-    
-    hData.Path.Size = retcode;
-    
-    /* Get info on object (reportID, offset and size) */
-    if (FindObject(&hParser,&hData) == 1) {
-      if (libusb_get_report(udev, hData.ReportID, rawbuf, REPORT_SIZE) > 0) { 
-	GetValue((const unsigned char *) rawbuf, &hData);
-
+	int r;
+	long hValue;  
+	
+	r = HIDGetItemLogical(udev, path, utab, &hValue, NULL);
+	if (r <= 0) {
+		return NULL;
+	}
+	
 	/* now get string */
-	libusb_get_string(udev, hData.Value, rawbuf);
+	comm_driver->get_string(udev, hValue, rawbuf);
 	return rawbuf;
-      }
-      else
-	TRACE(2, "Can't retrieve Report %i", hData.ReportID);
-    }
-    else
-      TRACE(2, "Can't find object %s", path);
-
-    return NULL;
-  }
-  return NULL;
 }
- 
-bool HIDSetItemValue(usb_dev_handle *udev, char *path, float value, usage_tables_t *utab)
+
+/* set the given physical value for the variable associated with
+ * path. Return TRUE on success, FALSE on failure. */ 
+bool HIDSetItemValue(hid_dev_handle *udev, char *path, float value, usage_tables_t *utab)
 {
 	float Value;
-	int retcode;
-	report_t *cur_report = &cur_report_struct;
+	int r;
+	long hValue, oldValue, newValue;
+	HIDData *pData;
+	int id;
 
-	/* Begin by a standard Get to fill in com structures ... */
-	retcode = HIDGetItemValue(udev, path, &Value, utab);
-	
-	/* ... And play with global vars */
-	if (retcode != 1) /* Get failed */
-	{
+	r = HIDGetItemLogical(udev, path, utab, &oldValue, &pData);
+	if (r <= 0) {
 		return FALSE;
 	}
 
-	TRACE(2, "=>> SET: Before set: %.2f (%ld)", Value, (long)value);
+	upsdebugx(4, "=>> SET: Before set: %ld (%ld)", oldValue, (long)value);
 	
+	id = pData->ReportID;
+
 	/* Test if Item is settable */
-	if (hData.Attribute == ATTR_DATA_CST) 
-	{
+        /* FIXME: not constant == volatile, but
+         * it doesn't imply that it's RW! */
+	if (pData->Attribute == ATTR_DATA_CST) {
 		return FALSE;
 	}
 
 	/* Set new value for this item */
+	/* And Process exponents restoration */
+	Value = value * expo(10, get_unit_expo(pData->Unit) - (int)pData->UnitExp);
 	
-	/* restore exponents */
-	value *= expo(10, get_unit_expo(hData.Unit) - (int)hData.UnitExp);
-	TRACE(2, "=>> SET: after exp: %.2f (exp = %.2f)", value,
-			expo(10, (int)get_unit_expo(hData.Unit) - (int)hData.UnitExp));
+	upsdebugx(4, "=>> SET: after exp: %.2f (exp = %.2f)", Value,
+	      expo(10, (int)get_unit_expo(pData->Unit) - (int)pData->UnitExp));
 	
 	/* convert physical value to logical */
-	hData.Value = physical_to_logical(&hData, value);
-	TRACE(2, "=>> SET: after PL: %ld", hData.Value);
+	hValue = physical_to_logical(pData, Value);
+	upsdebugx(4, "=>> SET: after PL: %ld", hValue);
 	
-	SetValue(&hData, cur_report->data);
-	
-	dump_hex ("==> Report after setvalue", cur_report->data, cur_report->len);
-	
-	if (libusb_set_report(udev, hData.ReportID, cur_report->data, cur_report->len) > 0)
-	{
-		TRACE(2, "Set report succeeded");
-		return TRUE;
-	}
-	else
-	{
-		TRACE(2, "Set report failed");
+	upsdebug_hex (4, "==> Report to set", rbuf->data[id], rbuf->len[id]);
+
+	r = set_item_buffered(rbuf, pData, pDesc, udev, hValue);
+	if (r<0) {
+		upsdebugx(2, "Set report failed: (%d): %s", errno, strerror(errno));
 		return FALSE;
 	}
-	/* check if set succeed! => doesn't work on *Delay (decremented!) */
-	/*      Value = HIDGetItemValue(path);
-			  
-	TRACE(2, "=>> SET: new value = %.2f (was set to %.2f)\n", 
-	Value, (float) value);
-	return TRUE;*/ /* (Value == value); */
+	
+	/* re-read report without buffering */
+	r = get_item_buffered(rbuf, pData, 0, pDesc, udev, &newValue);
+	if (r<0) {
+		upsdebugx(2, "Warning: Re-read report failed: (%d): %s", errno, strerror(errno));
+		return TRUE;
+	}
+	upsdebug_hex (4, "==> Report after set", rbuf->data[id], rbuf->len[id]);
+
+	if (newValue != hValue) {
+		/* this is normal; device will usually correct "out of
+		   range" values */
+		upsdebugx(4, "Wrote %ld, got %ld\n", hValue, newValue);
+	}
+
+	upsdebugx(4, "Set report succeeded");
+	return TRUE;
 }
 
-int HIDGetEvents(usb_dev_handle *udev, HIDDevice *dev, HIDItem **eventsList, usage_tables_t *utab)
+void HIDFreeEvents(HIDEvent *events) {
+	HIDEvent *p;
+
+	while (events) {
+		p = events->next; /* must copy this before free(events)! */
+		free(events->Path);
+		free(events);
+		events = p;
+	}
+}
+
+/* FIXME: change this so that we iterate through the report descriptor
+   once, instead of once for every offset. Simply pick out the items
+   with the correct ReportID and Type! On success, return item count
+   >=0 and set *eventsListp. On error, return <0. Note: on success, an
+   allocated events list will be returned in *eventsListp that must
+   later be freed by the caller using HIDFreeEvents(). */
+int HIDGetEvents(hid_dev_handle *udev, HIDDevice *dev, HIDEvent **eventsListp, usage_tables_t *utab)
 {
-	unsigned char buf[20];
+	unsigned char buf[100];
 	char itemPath[128];
-	int size, offset = 0, itemCount = 0;
-	
+	int size, itemCount;
+	long hValue;	
+	HIDData *pData;
+	int id, r, i;
+	HIDEvent *root = NULL;
+	HIDEvent **hook = &root;
+	HIDEvent *p;
+
 	upsdebugx(2, "Waiting for notifications...");
 	
 	/* needs libusb-0.1.8 to work => use ifdef and autoconf */
-	if ((size = libusb_get_interrupt(udev, &buf[0], REPORT_SIZE, 5000)) > -1)
+	if ((size = comm_driver->get_interrupt(udev, &buf[0], 100, 5000)) <= -1)
 	{
-		dump_hex ("Notification", buf, size);
-		
-		/* Convert report size in bits */
-		size = (size - 1) * 8;
-		
-		/* Parse response Report and Set correspondant Django values */
-		hData.ReportID = buf[0];
-		hData.Type = ITEM_INPUT;
-		
-		while(offset < size)
-		{
-			/* Set Offset */
-			hData.Offset = offset;
-	
-			/* Reset HID Path but keep Report ID */
-			memset(&hData.Path, '\0', sizeof(HIDPath));
-	
-			/* Get HID Object characteristics */
-			if(FindObject(&hParser, &hData))
-			{
-				/* Get HID Object value from report */
-				GetValue(buf, &hData);
-				memset(&itemPath, 0, sizeof(128));
-				path_to_string(itemPath, &hData.Path, utab);
-	
-				upsdebugx(3, "Object: %s = %ld", itemPath, hData.Value);
-	
-				/* FIXME: enhance this or fix/change the HID parser (see libhid project) */
-				/* if ( strstr(itemPath, "000000") == NULL) */
-				if (strcmp(itemPath, "UPS.PowerSummary.PresentStatus.") > 0)
-				{
-					eventsList[itemCount] = (HIDItem *)malloc(sizeof (HIDItem));
-					eventsList[itemCount]->Path = strdup(itemPath);
-					eventsList[itemCount]->Value = hData.Value;
-					itemCount++;
-				}
-			}
-			offset += hData.Size;
-		}
+		return size; /* propagate error code */
 	}
-	else
-		itemCount = size; /* propagate error code */
+	upsdebug_hex (3, "Notification", buf, size);
 
+	r = file_report_buffer(rbuf, buf, size);
+	if (r < 0) {
+		upsdebugx(2, "Failed to buffer report: %s", strerror(errno));
+		return -errno;
+	}
+
+	/* now read all items that are part of this report */
+	id = buf[0];
+
+	itemCount = 0;
+	for (i=0; i<pDesc->nitems; i++) {
+		pData = &pDesc->item[i];
+
+		if (pData->Type != ITEM_INPUT || pData->ReportID != id) {
+			continue;
+		}
+		path_to_string(itemPath, &pData->Path, utab);
+		GetValue(buf, pData, &hValue);
+		
+		upsdebugx(3, "Object: %s = %ld", itemPath, hValue);
+
+		/* FIXME: enhance this or fix/change the HID parser
+		   (see libhid project) */
+		p = (HIDEvent *)malloc(sizeof (HIDEvent));
+		if (!p) {
+			HIDFreeEvents(root);
+			return -errno;
+		}
+		p->Path = strdup(itemPath);
+		p->Value = hValue;
+		p->next = NULL;
+		*hook = p;
+		hook = &p->next;
+		itemCount++;
+	}
+	*eventsListp = root;
 	return itemCount;
 }
 
-void HIDCloseDevice(usb_dev_handle *udev)
+void HIDCloseDevice(hid_dev_handle *udev)
 {
-	if (udev != NULL)
-	{
-		TRACE(2, "Closing device");
-		libusb_close(udev);
-	}
+    if (udev != NULL)
+    {
+      upsdebugx(2, "Closing device");
+      comm_driver->close(udev);
+    }
 }
 
 
@@ -741,7 +902,7 @@ static long physical_to_logical(HIDData *Data, float physical)
 {
 	long logical, Factor;
 
-	TRACE(2, "PhyMax = %ld, PhyMin = %ld, LogMax = %ld, LogMin = %ld",
+	upsdebugx(4, "PhyMax = %ld, PhyMin = %ld, LogMax = %ld, LogMin = %ld",
 		Data->PhyMax, Data->PhyMin, Data->LogMax, Data->LogMin);
 	
 	/* HID spec says that if one or both are undefined, or if they are
@@ -801,7 +962,7 @@ static float expo(int a, int b)
 	return -1;
 }
 
-/* translate HID string path from/to numeric path and return path depth */
+/* translate HID string path to numeric path and return path depth */
 /* TODO: use usbutils functions (need to be externalised!) */
 static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 {
@@ -810,7 +971,7 @@ static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 	char buf[MAX_STRING];
 	char *start, *end; 
 	
-	TRACE(3, "entering string_to_path()");
+	upsdebugx(5, "entering string_to_path()");
 	
 	strncpy(buf, HIDpath, min(strlen(HIDpath)+1, MAX_STRING));
 	buf[MAX_STRING-1] = '\0';
@@ -824,16 +985,15 @@ static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 		else
 			*end = '\0';
 		
-		TRACE(4, "parsing %s", start);
+		upsdebugx(4, "parsing %s", start);
 		
 		/* lookup code */
 		if ((cur_usage = hid_lookup_usage(start, utab)) == -1) {
-			TRACE(4, "%s wasn't found", start);
+			upsdebugx(4, "%s wasn't found", start);
 			return 0;
 		}
 		else {
-			path->Node[i].UPage = (cur_usage & 0xFFFF0000) / 0x10000;
-			path->Node[i].Usage = cur_usage & 0x0000FFFF; 
+			path->Node[i] = cur_usage;
 			i++;
 		}
 		
@@ -845,32 +1005,32 @@ static int string_to_path(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 	return i;
 }
 
-/* translate HID string path from/to numeric path and return path depth */
+/* translate HID numeric path to string path and return path depth */
 /* TODO: use usbutils functions (need to be externalised!) */
+/* FIXME: char *HIDpath must be large enough; buffer overflow otherwise. */
 static int path_to_string(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 {
 	int i = 0;
 	
-	TRACE(3, "entering path_to_string()");
+	upsdebugx(5, "entering path_to_string()");
 	
-	/* FIXME: another bug? */
-	strcat(HIDpath, "UPS.");
+	HIDpath[0] = '\0';
 	
-	// Numeric to String
-	for (i = 1; i < path->Size; i++)
+	/* Numeric to String */
+	for (i = 0; i < path->Size; i++)
 	{
 		/* Deal with ?bogus? */
-		if (path->Node[i].UPage == 0 && path->Node[i].Usage == 0)
+		if (path->Node[i] == 0)
 			continue;
 		
 		/* manage indexed collection */
-		if (path->Node[i].UPage == 0x00FF)
+		if ((path->Node[i] & 0xffff0000) == 0x00ff0000)
 		{
-			TRACE(5, "Got an indexed collection");
-			sprintf(strrchr(HIDpath, '.'), "[%i]", path->Node[i].Usage);
+			upsdebugx(5, "Got an indexed collection");
+			sprintf(strrchr(HIDpath, '.'), "[%i]", path->Node[i] & 0x0000ffff);
 		}
 		else
-			strcat(HIDpath, hid_lookup_path((path->Node[i].UPage * 0x10000) + path->Node[i].Usage, utab));
+			strcat(HIDpath, hid_lookup_path(path->Node[i], utab));
 		
 		if (i < (path->Size - 1))
 			strcat (HIDpath, ".");
@@ -888,42 +1048,77 @@ static int path_to_string(char *HIDpath, HIDPath *path, usage_tables_t *utab)
 usage_lkp_t hid_usage_lkp[] = {
 	/* Power Device Page */
 	{  "Undefined",				0x00840000 },
+	{  "iName",				0x00840001 },
 	{  "PresentStatus",			0x00840002 },
+	{  "ChangedStatus",			0x00840003 },
 	{  "UPS",				0x00840004 },
+	{  "PowerSupply",			0x00840005 },
+	/* 0x00840006-0x0084000f	=>	Reserved */
 	{  "BatterySystem",			0x00840010 },
+	{  "BatterySystemID",			0x00840011 },
 	{  "Battery",				0x00840012 },
 	{  "BatteryID",				0x00840013 },
+	{  "Charger",				0x00840014 },
+	{  "ChargerID",				0x00840015 },
 	{  "PowerConverter",			0x00840016 },
 	{  "PowerConverterID",			0X00840017 },
 	{  "OutletSystem",			0x00840018 },
 	{  "OutletSystemID",			0x00840019 },
 	{  "Input",				0x0084001a },
+	{  "InputID",				0x0084001b },
 	{  "Output",				0x0084001c },
+	{  "OutputID",				0x0084001d },
 	{  "Flow",				0x0084001e },
 	{  "FlowID",				0x0084001f },
 	{  "Outlet",				0x00840020 },
 	{  "OutletID",				0x00840021 },
+	{  "Gang",				0x00840022 },
+	{  "GangID",				0x00840023 },
 	{  "PowerSummary",			0x00840024 },
 	{  "PowerSummaryID",			0x00840025 },
+	/* 0x00840026-0x0084002f	=>	Reserved */
 	{  "Voltage",				0x00840030 },
 	{  "Current",				0x00840031 },
 	{  "Frequency",				0x00840032 },
+	{  "ApparentPower",			0x00840033 },
+	{  "ActivePower",			0x00840034 },
 	{  "PercentLoad",			0x00840035 },
 	{  "Temperature",			0x00840036 },
+	{  "Humidity",				0x00840037 },
+	{  "BadCount",				0x00840038 },
+	/* 0x00840039-0x0084003f	=>	Reserved */
 	{  "ConfigVoltage",			0x00840040 },
 	{  "ConfigCurrent",			0x00840041 },
 	{  "ConfigFrequency",			0x00840042 },
 	{  "ConfigApparentPower",		0x00840043 },
+	{  "ConfigActivePower",			0x00840044 },
+	{  "ConfigPercentLoad",			0x00840045 },
+	{  "ConfigTemperature",			0x00840046 },
+	{  "ConfigHumidity",			0x00840047 },
+	/* 0x00840048-0x0084004f	=>	Reserved */
+	{  "SwitchOnControl",			0x00840050 },
+	{  "SwitchOffControl",			0x00840051 },
+	{  "ToggleControl",			0x00840052 },
 	{  "LowVoltageTransfer",		0x00840053 },
 	{  "HighVoltageTransfer",		0x00840054 },	
 	{  "DelayBeforeReboot",			0x00840055 },
 	{  "DelayBeforeStartup",		0x00840056 },
 	{  "DelayBeforeShutdown",		0x00840057 },
 	{  "Test",				0x00840058 },
+	{  "ModuleReset",			0x00840059 },
 	{  "AudibleAlarmControl",		0x0084005a },
+	/* 0x0084005b-0x0084005f	=>	Reserved */
+	{  "Present",				0x00840060 },
 	{  "Good",				0x00840061 },
 	{  "InternalFailure",			0x00840062 },
-	{  "OverLoad",				0x00840065 }, /* mispelled in usb.ids */
+	{  "VoltageOutOfRange",			0x00840063 },
+	{  "FrequencyOutOfRange",		0x00840064 },
+	{  "Overload",				0x00840065 }, 
+        /* Note: the correct spelling is "Overload", not "OverLoad",
+	 * according to the official specification, "Universal Serial
+	 * Bus Usage Tables for HID Power Devices", Release 1.0,
+	 * November 1, 1997 */
+	{  "OverCharged",			0x00840066 },
 	{  "OverTemperature", 			0x00840067 },
 	{  "ShutdownRequested",			0x00840068 },
 	{  "ShutdownImminent",			0x00840069 },
@@ -932,39 +1127,120 @@ usage_lkp_t hid_usage_lkp[] = {
 	{  "Used",				0x0084006d },
 	{  "Boost",				0x0084006e },
 	{  "Buck",				0x0084006f },
+	{  "Initialized",			0x00840070 },
+	{  "Tested",				0x00840071 },
+	{  "AwaitingPower",			0x00840072 },
 	{  "CommunicationLost",			0x00840073 },
+	/* 0x00840074-0x008400fc	=>	Reserved */
 	{  "iManufacturer",			0x008400fd },
 	{  "iProduct",				0x008400fe },
 	{  "iSerialNumber",			0x008400ff },
+
 	/* Battery System Page */
 	{ "Undefined",				0x00850000 },
+	{ "SMBBatteryMode",			0x00850001 },
+	{ "SMBBatteryStatus",			0x00850002 },
+	{ "SMBAlarmWarning",			0x00850003 },
+	{ "SMBChargerMode",			0x00850004 },
+	{ "SMBChargerStatus",			0x00850005 },
+	{ "SMBChargerSpecInfo",			0x00850006 },
+	{ "SMBSelectorState",			0x00850007 },
+	{ "SMBSelectorPresets",			0x00850008 },
+	{ "SMBSelectorInfo",			0x00850009 },
+	/* 0x0085000A-0x0085000f	=>	Reserved */
+	{ "OptionalMfgFunction1",		0x00850010 },
+	{ "OptionalMfgFunction2",		0x00850011 },
+	{ "OptionalMfgFunction3",		0x00850012 },
+	{ "OptionalMfgFunction4",		0x00850013 },
+	{ "OptionalMfgFunction5",		0x00850014 },
+	{ "ConnectionToSMBus",			0x00850015 },
+	{ "OutputConnection",			0x00850016 },
+	{ "ChargerConnection",			0x00850017 },
+	{ "BatteryInsertion",			0x00850018 },
+	{ "Usenext",				0x00850019 },
+	{ "OKToUse",				0x0085001a },
+	{ "BatterySupported",			0x0085001b },
+	{ "SelectorRevision",			0x0085001c },
+	{ "ChargingIndicator",			0x0085001d },
+	/* 0x0085001e-0x00850027	=>	Reserved */
+	{ "ManufacturerAccess",			0x00850028 },
 	{ "RemainingCapacityLimit",		0x00850029 },
 	{ "RemainingTimeLimit",			0x0085002a },
+	{ "AtRate",				0x0085002b },
 	{ "CapacityMode",			0x0085002c },
+	{ "BroadcastToCharger",			0x0085002d },
+	{ "PrimaryBattery",			0x0085002e },
+	{ "ChargeController",			0x0085002f },
+	/* 0x00850030-0x0085003f	=>	Reserved */
+	{ "TerminateCharge",			0x00850040 },
+	{ "TerminateDischarge",			0x00850041 },
 	{ "BelowRemainingCapacityLimit",	0x00850042 },
 	{ "RemainingTimeLimitExpired",		0x00850043 },
 	{ "Charging",				0x00850044 },
 	{ "Discharging",			0x00850045 },
 	{ "FullyCharged",			0x00850046 },
 	{ "FullyDischarged",			0x00850047 },
+	{ "ConditioningFlag",			0x00850048 },
+	{ "AtRateOK",				0x00850049 },
+	{ "SMBErrorCode",			0x0085004a },
 	{ "NeedReplacement",			0x0085004b },
+	/* 0x0085004c-0x0085005f	=>	Reserved */
+	{ "AtRateTimeToFull",			0x00850060 },
+	{ "AtRateTimeToEmpty",			0x00850061 },
+	{ "AverageCurrent",			0x00850062 },
+	{ "Maxerror",				0x00850063 },
+	{ "RelativeStateOfCharge",		0x00850064 },
+	{ "AbsoluteStateOfCharge",		0x00850065 },
 	{ "RemainingCapacity",			0x00850066 },
 	{ "FullChargeCapacity",			0x00850067 },
 	{ "RunTimeToEmpty",			0x00850068 },
+	{ "AverageTimeToEmpty",			0x00850069 },
+	{ "AverageTimeToFull",			0x0085006a },
+	{ "CycleCount",				0x0085006b },
+	/* 0x0085006c-0x0085007f	=>	Reserved */
+	{ "BattPackModelLevel",			0x00850080 },
+	{ "InternalChargeController",		0x00850081 },
+	{ "PrimaryBatterySupport",		0x00850082 },
+	{ "DesignCapacity",			0x00850083 },
+	{ "SpecificationInfo",			0x00850084 },
 	{ "ManufacturerDate",			0x00850085 },
+	{ "SerialNumber",			0x00850086 },
+	{ "iManufacturerName",			0x00850087 },
+	{ "iDevicename",			0x00850088 }, /* sic! */
+	{ "iDeviceChemistry",			0x00850089 }, /* misspelled as "iDeviceChemistery" in spec. */
+	{ "ManufacturerData",			0x0085008a },
 	{ "Rechargeable",			0x0085008b },
 	{ "WarningCapacityLimit",		0x0085008c },
 	{ "CapacityGranularity1",		0x0085008d },
 	{ "CapacityGranularity2",		0x0085008e },
 	{ "iOEMInformation",			0x0085008f },
-	{ "DesignCapacity",			0x00850083 },
-	{ "iDeviceChemistry",			0x00850089 },
+	/* 0x00850090-0x008500bf	=>	Reserved */
+	{ "InhibitCharge",			0x008500c0 },
+	{ "EnablePolling",			0x008500c1 },
+	{ "ResetToZero",			0x008500c2 },
+	/* 0x008500c3-0x008500cf	=>	Reserved */
 	{ "ACPresent",				0x008500d0 },
 	{ "BatteryPresent",			0x008500d1 },
+	{ "PowerFail",				0x008500d2 },
+	{ "AlarmInhibited",			0x008500d3 },
+	{ "ThermistorUnderRange",		0x008500d4 },
+	{ "ThermistorHot",			0x008500d5 },
+	{ "ThermistorCold",			0x008500d6 },
+	{ "ThermistorOverRange",		0x008500d7 },
+	{ "VoltageOutOfRange",			0x008500d8 },
+	{ "CurrentOutOfRange",			0x008500d9 },
+	{ "CurrentNotRegulated",		0x008500da },
 	{ "VoltageNotRegulated",		0x008500db },
+	{ "MasterMode",				0x008500dc },
+	/* 0x008500dd-0x008500ef	=>	Reserved */
+	{ "ChargerSelectorSupport",		0x008500f0 },
+	{ "ChargerSpec",			0x008500f1 },
+	{ "Level2",				0x008500f2 },
+	{ "Level3",				0x008500f3 },
+	/* 0x008500f4-0x008500ff	=>	Reserved */
 
 	/* end of structure. */
-	{  "\0", 0x0 }
+	{  "\0",				0x00000000 }
 };
 
 /* usage conversion numeric -> string */
@@ -974,7 +1250,7 @@ static const char *hid_lookup_path(unsigned int usage, usage_tables_t *utab)
 	static char raw_usage[10];
 	usage_lkp_t *table;
 
-	TRACE(3, "Looking up %08x", usage);
+	upsdebugx(5, "Looking up %08x", usage);
 
 	for (j=0; utab[j] != NULL; j++) {
 		table = utab[j];
@@ -999,7 +1275,7 @@ static int hid_lookup_usage(char *name, usage_tables_t *utab)
 	char buf[20];
 	usage_lkp_t *table;
 
-	TRACE(3, "Looking up %s", name);
+	upsdebugx(5, "Looking up %s", name);
 	
 	if (name[0] == '[') { /* manage indexed collection */
 		return (0x00FF0000 + atoi(&name[1]));
@@ -1010,7 +1286,7 @@ static int hid_lookup_usage(char *name, usage_tables_t *utab)
 		{
 			if (!strcmp(table[i].usage_name, name))
 			{
-				TRACE(4, "hid_lookup_usage: found %04x",
+				upsdebugx(4, "hid_lookup_usage: found %04x",
 				      table[i].usage_code);
 				
 				return table[i].usage_code;
@@ -1027,47 +1303,3 @@ static int hid_lookup_usage(char *name, usage_tables_t *utab)
 	return value;
 }
 
-int get_current_data_attribute()
-{
-	return hData.Attribute;
-}
-#define NIBBLE(_i)    (((_i) < 10) ? '0' + (_i) : 'A' + (_i) - 10)
-
-static void dump_hex (const char *msg, const unsigned char *buf, int len)
-{
-	int i;
-	int nlocal;
-	const unsigned char *pc;
-	char *out;
-	const unsigned char *start;
-	char c;
-	char line[100];
- 
-	start = buf;
-	out = line;
-	
-	for (i = 0, pc = buf, nlocal = len; i < 16; i++, pc++)
-	{
-		if (nlocal > 0)
-		{
-			c = *pc;
-
-			*out++ = NIBBLE ((c >> 4) & 0xF);
-			*out++ = NIBBLE (c & 0xF);
-
-			nlocal--;
-		}
-		else
-		{
-			*out++ = ' ';
-			*out++ = ' ';
-		}
-		*out++ = ' ';
-	}
-	*out++ = 0;
-
-	TRACE(3, "%s: (%d bytes) => %s", msg, len, line);
-
-	buf += 16;
-	len -= 16;
-}
