@@ -535,13 +535,16 @@ int main(int argc, char **argv)
 
 	pidfn = xmalloc(SMALLBUF);
 
-	snprintf(pidfn, SMALLBUF, "%s/%s-%s.pid", 
-		altpidpath(), progname, device_name);
+	if (upsname_found)
+		snprintf(pidfn, SMALLBUF, "%s/%s-%s.pid", 
+			altpidpath(), progname, upsname);
+	else
+		snprintf(pidfn, SMALLBUF, "%s/%s-%s.pid", 
+			 altpidpath(), progname, device_name);
 
 	upsdebugx(1, "debug level is '%d'", nut_debug_level);
 
-	if ((new_uid = get_user_pwent(user)) == NULL)
-		fatal("getpwnam(%s)", user);
+	new_uid = get_user_pwent(user);
 	
 	if (chroot_path)
 		chroot_start(chroot_path);
@@ -552,7 +555,7 @@ int main(int argc, char **argv)
 	/* This avoid case where ie /var is umounted */
 	if (!do_forceshutdown)
 		if (chdir(dflt_statepath()))
-			fatal("Can't chdir to %s", dflt_statepath());
+			fatal_with_errno("Can't chdir to %s", dflt_statepath());
 
 	setup_signals();
 
@@ -576,7 +579,10 @@ int main(int argc, char **argv)
 	upsdrv_updateinfo();
 
 	/* now we can start servicing requests */
-	dstate_init(progname, device_name);
+	if (upsname_found)
+		dstate_init(upsname, NULL);
+	else
+		dstate_init(progname, device_name);
 
 	/* publish the top-level data: version number, driver name */
 	dstate_setinfo("driver.version", "%s", UPS_VERSION);
