@@ -1,7 +1,7 @@
 /*
  * powercom.h - defines for the newpowercom.c driver
  *
- * $Id: powercom.h 8 2005-02-28 09:14:07Z  $
+ * $Id: powercom.h 569 2006-11-08 02:08:45Z selinger-guest $
  *
  * Copyrights:
  * (C) 2002 Simon Rozman <simon@rozman.net>
@@ -28,7 +28,7 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
-#include <sys/termios.h>
+#include "serial.h"
 #include <limits.h>
 
 /* nut includes */
