@@ -106,6 +106,11 @@ typedef struct {
 #define SU_FLAG_ABSENT		(1 << 2)		/* data is absent in the device, */
 										/* use default value. */
 #define SU_FLAG_STALE		(1 << 3)		/* data stale, don't try too often. */
+#define SU_FLAG_NEGINVALID	(1 << 4)	/* Invalid if negative value */
+#define SU_FLAG_UNIQUE		(1 << 5)	/* There can be only be one
+						 * provider of this info,
+						 * disable the other providers
+						 */
 
 /* status string components */
 #define SU_STATUS_PWR		(0 << 8)		/* indicates power status element */
@@ -153,7 +158,7 @@ typedef struct {
 void nut_snmp_init(const char *type, const char *host, const char *community);
 void nut_snmp_cleanup(void);
 struct snmp_pdu *nut_snmp_get(const char *OID);
-bool nut_snmp_get_str(const char *OID, char *buf, size_t buf_len);
+bool nut_snmp_get_str(const char *OID, char *buf, size_t buf_len, info_lkp_t *oid2info);
 bool nut_snmp_get_int(const char *OID, long *pval);
 bool nut_snmp_set(const char *OID, char type, const char *value);
 bool nut_snmp_set_str(const char *OID, const char *value);
