@@ -41,7 +41,7 @@ typedef struct {
 
 static	ups_t	*upstable = NULL;
 
-static	int	verbose = 0, maxsdorder = 0, testmode = 0, exec_error = 0;
+static	int	maxsdorder = 0, testmode = 0, exec_error = 0;
 
 	/* timer - keeps us from getting stuck if a driver hangs */
 static	int	maxstartdelay = 45;
@@ -82,10 +82,12 @@ void do_upsconf_args(char *upsname, char *var, char *val)
 		last = tmp;
 
 		if (!strcmp(tmp->upsname, upsname)) {
-			if (!strcmp(var, "driver")) 
+			if (!strcmp(var, "driver"))
 				tmp->driver = xstrdup(val);
-			if (!strcmp(var, "port")) 
+
+			if (!strcmp(var, "port"))
 				tmp->port = xstrdup(val);
+
 			if (!strcmp(var, "maxstartdelay"))
 				tmp->maxstartdelay = atoi(val);
 
@@ -112,6 +114,7 @@ void do_upsconf_args(char *upsname, char *var, char *val)
 
 	if (!strcmp(var, "driver"))
 		tmp->driver = xstrdup(val);
+
 	if (!strcmp(var, "port"))
 		tmp->port = xstrdup(val);
 
@@ -122,102 +125,51 @@ void do_upsconf_args(char *upsname, char *var, char *val)
 }
 
 /* handle sending the signal */
-static void send_term(char *upsname, char *driver, char *port)
+static void stop_driver(const ups_t *ups)
 {
-	char	pidfn[SMALLBUF], buf[SMALLBUF];
-	int	ret, pid;
+	char	pidfn[SMALLBUF];
+	int	ret;
 	struct	stat	fs;
-	FILE	*pidf;
 
-	printf("Stopping UPS: %s\n", upsname);
+	upsdebugx(1, "Stopping UPS: %s", ups->upsname);
 
 	snprintf(pidfn, sizeof(pidfn), "%s/%s-%s.pid", altpidpath(),
-		driver, xbasename(port));
+		ups->driver, xbasename(ups->port));
 	ret = stat(pidfn, &fs);
 
 	if (ret != 0) {
-		upslog(LOG_ERR, "Can't open %s", pidfn);
+		snprintf(pidfn, sizeof(pidfn), "%s/%s-%s.pid", altpidpath(),
+			ups->driver, ups->upsname);
+		ret = stat(pidfn, &fs);
+	}
+
+	if (ret != 0) {
+		upslog_with_errno(LOG_ERR, "Can't open %s", pidfn);
 		exec_error++;
 		return;
 	}
 
-	pidf = fopen(pidfn, "r");
-
-	if (!pidf) {
-		upslog(LOG_ERR, "Can't open %s", pidfn);
-		exec_error++;
-		return;
-	}
-
-	fgets(buf, sizeof(buf), pidf);
-	buf[strlen(buf)-1] = '\0';
-
-	pid = strtol(buf, (char **)NULL, 10);
-
-	if (pid < 2) {
-		upslogx(LOG_NOTICE, "Ignoring invalid pid %d in %s",
-			pid, pidfn);
-		exec_error++;
-		return;
-	}
-
-	if (verbose)
-		printf("Sending signal: kill -TERM %d\n", pid);
+	upsdebugx(2, "Sending signal to %s", pidfn);
 
 	if (testmode)
 		return;
 
-	ret = kill(pid, SIGTERM);
+	ret = sendsignalfn(pidfn, SIGTERM);
 
 	if (ret < 0) {
-		upslog(LOG_ERR, "kill pid %d failed", pid);
+		upslog_with_errno(LOG_ERR, "Stopping %s failed", pidfn);
 		exec_error++;
 		return;
 	}
 }
 
-/* stop user-selected driver */
-static void stop_one_driver(char *upsname)
-{
-	ups_t	*tmp = upstable;
-
-	if (!tmp)
-		fatalx("Error: no UPS definitions found in ups.conf!\n");
-
-	while (tmp) {
-		if (!strcmp(tmp->upsname, upsname)) {
-			send_term(tmp->upsname, tmp->driver, tmp->port);
-			return;
-		}
-
-		tmp = tmp->next;
-	}
-
-	fatalx("UPS %s not found in ups.conf", upsname);
-}
-
-/* walk ups table, but stop drivers instead */
-static void stop_all_drivers(void)
-{
-	ups_t	*tmp = upstable;
-
-	if (!tmp)
-		fatalx("Error: no UPS definitions found in ups.conf!\n");
-
-	while (tmp) {
-		send_term(tmp->upsname, tmp->driver, tmp->port);
-
-		tmp = tmp->next;
-	}
-}
-
-static void waitpid_timeout(int sig)
+static void waitpid_timeout(const int sig)
 {
 	/* do nothing */
 	return;
 }
 
-static void forkexec(const char *prog, char **argv, ups_t *ups)
+static void forkexec(const char *prog, char **argv, const ups_t *ups)
 {
 	int	ret;
 	pid_t	pid;
@@ -225,7 +177,7 @@ static void forkexec(const char *prog, char **argv, ups_t *ups)
 	pid = fork();
 
 	if (pid < 0)
-		fatal("fork");
+		fatal_with_errno("fork");
 
 	if (pid != 0) {			/* parent */
 		int	wstat;
@@ -268,7 +220,7 @@ static void forkexec(const char *prog, char **argv, ups_t *ups)
 		/* the rest only work when WIFEXITED is nonzero */
 
 		if (WIFSIGNALED(wstat)) {
-			upslog(LOG_WARNING, "Driver died after signal %d",
+			upslog_with_errno(LOG_WARNING, "Driver died after signal %d",
 				WTERMSIG(wstat));
 			exec_error++;
 		}
@@ -281,26 +233,26 @@ static void forkexec(const char *prog, char **argv, ups_t *ups)
 	ret = execv(prog, argv);
 
 	/* shouldn't get here */
-	fatal("execv");
-}		
+	fatal_with_errno("execv");
+}
 
-static void start_driver(ups_t *ups)
+static void start_driver(const ups_t *ups)
 {
 	char	dfn[SMALLBUF], *argv[8];
 	int	ret, arg = 0;
 	struct	stat	fs;
 
+	upsdebugx(1, "Starting UPS: %s", ups->upsname);
+
 	snprintf(dfn, sizeof(dfn), "%s/%s", driverpath, ups->driver);
 	ret = stat(dfn, &fs);
 
 	if (ret < 0)
-		fatal("Can't start %s", dfn);
+		fatal_with_errno("Can't start %s", dfn);
 
-	if (verbose)
-	    	printf("exec: %s -a %s", dfn, ups->upsname);
-
-	argv[arg++] = xstrdup(dfn);
+	upsdebugx(2, "exec: %s -a %s", dfn, ups->upsname);
 
+	argv[arg++] = dfn;
 	argv[arg++] = "-a";
 	argv[arg++] = ups->upsname;
 
@@ -308,23 +260,15 @@ static void start_driver(ups_t *ups)
 	if (pt_root) {
 		argv[arg++] = "-r";
 		argv[arg++] = pt_root;
-
-		if (verbose)
-			printf(" -r %s", pt_root);
 	}
 
 	if (pt_user) {
 		argv[arg++] = "-u";
 		argv[arg++] = pt_user;
-
-		if (verbose)
-			printf(" -u %s", pt_user);
 	}
 
-	if (verbose)
-		printf("\n");
 	if (testmode)
-	    	return;
+		return;
 
 	/* tie it off */
 	argv[arg++] = NULL;
@@ -332,40 +276,6 @@ static void start_driver(ups_t *ups)
 	forkexec(dfn, argv, ups);
 }
 
-/* start user-selected driver */
-static void start_one_driver(char *upsname)
-{
-	ups_t	*tmp = upstable;
-
-	if (!tmp)
-		fatalx("Error: no UPS definitions found in ups.conf!\n");
-
-	while (tmp) {
-		if (!strcmp(tmp->upsname, upsname)) {
-			start_driver(tmp);
-			return;
-		}
-
-		tmp = tmp->next;
-	}
-
-	fatalx("UPS %s not found in ups.conf", upsname);
-}
-
-/* walk ups table and invoke drivers */
-static void start_all_drivers(void)
-{
-	ups_t	*tmp = upstable;
-
-	if (!tmp)
-		fatalx("Error: no UPS definitions found in ups.conf!\n");
-
-	while (tmp) {
-		start_driver(tmp);
-		tmp = tmp->next;
-	}
-}
-
 static void help(const char *progname)
 {
 	printf("Starts and stops UPS drivers via ups.conf.\n\n");
@@ -375,7 +285,7 @@ static void help(const char *progname)
 	printf("  -r <path>		drivers will chroot to <path>\n");
 	printf("  -t			testing mode - prints actions without doing them\n");
 	printf("  -u <user>		drivers started will switch from root to <user>\n");
-	printf("  -v			enable verbose messages\n");
+	printf("  -D            	raise debugging level\n");
 	printf("  start			start all UPS drivers in ups.conf\n");
 	printf("  start	<ups>		only start driver for UPS <ups>\n");
 	printf("  stop			stop all UPS drivers in ups.conf\n");
@@ -386,62 +296,77 @@ static void help(const char *progname)
 	exit(EXIT_SUCCESS);
 }
 
-static void shutdown_driver(ups_t *ups)
+static void shutdown_driver(const ups_t *ups)
 {
 	char	*argv[7], dfn[SMALLBUF];
+	int	arg = 0;
+
+	upsdebugx(1, "Shutdown UPS: %s", ups->upsname);
 
 	snprintf(dfn, sizeof(dfn), "%s/%s", driverpath, ups->driver);
 
-	if (verbose)
-		printf("exec: %s -a %s -k\n", dfn, ups->upsname);
-	if (testmode)
-	    	return;
+	upsdebugx(2, "exec: %s -a %s -k", dfn, ups->upsname);
 
-	argv[0] = dfn;
-	argv[1] = "-a";
-	argv[2] = xstrdup(ups->upsname);
-	argv[3] = "-k";
-	argv[4] = NULL;
+	if (testmode)
+		return;
+
+	argv[arg++] = dfn;
+	argv[arg++] = "-a";
+	argv[arg++] = ups->upsname;
+	argv[arg++] = "-k";
+	argv[arg++] = NULL;
 
 	forkexec(dfn, argv, ups);
 }
 
-static void shutdown_one_driver(char *upsname)
+static void send_one_driver(void (*command)(const ups_t *), const char *upsname)
 {
-	ups_t	*tmp = upstable;
+	ups_t	*ups = upstable;
 
-	if (!tmp)
+	if (!ups)
 		fatalx("Error: no UPS definitions found in ups.conf!\n");
 
-	while (tmp) {
-		if (!strcmp(tmp->upsname, upsname)) {
-			shutdown_driver(tmp);
+	while (ups) {
+		if (!strcmp(ups->upsname, upsname)) {
+			command(ups);
 			return;
 		}
 
-		tmp = tmp->next;
+		ups = ups->next;
 	}
 
 	fatalx("UPS %s not found in ups.conf", upsname);
 }
 
-/* walk UPS table and shut down all UPSes according to sdorder */
-static void shutdown_all_drivers(void)
+/* walk UPS table and send command to all UPSes according to sdorder */
+static void send_all_drivers(void (*command)(const ups_t *))
 {
-	ups_t	*tmp;
+	ups_t	*ups;
 	int	i;
 
 	if (!upstable)
 		fatalx("Error: no UPS definitions found in ups.conf");
 
-	for (i = 0; i <= maxsdorder; i++) {
-		tmp = upstable;
+	if (command != &shutdown_driver) {
+		ups = upstable;
 
-		while (tmp) {
-			if (tmp->sdorder == i)
-				shutdown_driver(tmp);
+		while (ups) {
+			command(ups);
+
+			ups = ups->next;
+		}
+
+		return;
+	}
+
+	for (i = 0; i <= maxsdorder; i++) {
+		ups = upstable;
+
+		while (ups) {
+			if (ups->sdorder == i)
+				command(ups);
 			
-			tmp = tmp->next;
+			ups = ups->next;
 		}
 	}
 }
@@ -457,8 +382,10 @@ static void exit_cleanup(void)
 
 		if (tmp->driver)
 			free(tmp->driver);
+
 		if (tmp->port)
 			free(tmp->port);
+
 		if (tmp->upsname)
 			free(tmp->upsname);
 		free(tmp);
@@ -474,12 +401,13 @@ int main(int argc, char **argv)
 {
 	int	i;
 	char	*prog;
+	void	(*command)(const ups_t *) = NULL;
 
 	printf("Network UPS Tools - UPS driver controller %s\n",
 		UPS_VERSION);
 
 	prog = argv[0];
-	while ((i = getopt(argc, argv, "+htvu:r:V")) != EOF) {
+	while ((i = getopt(argc, argv, "+htu:r:DV")) != EOF) {
 		switch(i) {
 			case 'r':
 				pt_root = optarg;
@@ -487,24 +415,19 @@ int main(int argc, char **argv)
 
 			case 't':
 				testmode = 1;
-
-				/* force verbose mode while testing */
-				if (verbose == 0)
-					verbose = 1;
-
 				break;
 
 			case 'u':
 				pt_user = optarg;
 				break;
 
-			case 'v':
-				verbose++;
-				break;
-
 			case 'V':
 				exit(EXIT_SUCCESS);
 
+			case 'D':
+				nut_debug_level++;
+				break;
+
 			case 'h':
 			default:
 				help(prog);
@@ -518,57 +441,38 @@ int main(int argc, char **argv)
 	if (argc < 1)
 		help(prog);
 
-	if (testmode)
+	if (testmode) {
 		printf("*** Testing mode: not calling exec/kill\n");
 
-	driverpath = xstrdup(DRVPATH);  /* set default */
+		if (nut_debug_level < 2)
+			nut_debug_level = 2;
+	}
+
+	if (!strcmp(argv[0], "start"))
+		command = &start_driver;
+
+	if (!strcmp(argv[0], "stop"))
+		command = &stop_driver;
+
+	if (!strcmp(argv[0], "shutdown"))
+		command = &shutdown_driver;
+
+	if (!command)
+		fatalx("Error: unrecognized command [%s]", argv[0]);
+
+	driverpath = xstrdup(DRVPATH);	/* set default */
 
 	atexit(exit_cleanup);
 
-	if (!strcmp(argv[0], "start")) {
-		read_upsconf();
+	read_upsconf();
 
-		if (argc == 1)
-			start_all_drivers();
-		else
-			start_one_driver(argv[1]);
+	if (argc == 1)
+		send_all_drivers(command);
+	else
+		send_one_driver(command, argv[1]);
 
-		if (exec_error)
-			exit(EXIT_FAILURE);
+	if (exec_error)
+		exit(EXIT_FAILURE);
 
-		exit(EXIT_SUCCESS);
-	}
-
-	if (!strcmp(argv[0], "stop")) {
-		read_upsconf();
-
-		if (argc == 1)
-			stop_all_drivers();
-		else
-			stop_one_driver(argv[1]);
-
-		if (exec_error)
-			exit(EXIT_FAILURE);
-
-		exit(EXIT_SUCCESS);
-	}
-
-	if (!strcmp(argv[0], "shutdown")) {
-		read_upsconf();
-
-		if (argc == 1)
-			shutdown_all_drivers();
-		else
-			shutdown_one_driver(argv[1]);
-
-		if (exec_error)
-			exit(EXIT_FAILURE);
-
-		exit(EXIT_SUCCESS);
-	}
-
-	fatalx("Error: unrecognized command [%s]\n", argv[0]);
-
-	/* NOTREACHED */
-	exit(EXIT_FAILURE);
+	exit(EXIT_SUCCESS);
 }
