@@ -20,6 +20,12 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+
+/* Need this on AIX when using xlc to get alloca */
+#ifdef _AIX
+#pragma alloca
+#endif /* _AIX */
+
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -58,17 +64,18 @@ void chroot_start(const char *path);
 void writepid(const char *name);
 
 /* send a signal to another running process */
-void sendsignal(const char *progname, int sig);
+int sendsignal(const char *progname, int sig);
 
 int snprintfcat(char *dst, size_t size, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 3, 4)));
 
 /* open <pidfn>, get the pid, then send it <sig> */
-void sendsignalfn(const char *pidfn, int sig);
+int sendsignalfn(const char *pidfn, int sig);
 
 const char *xbasename(const char *file);
 
-/* enable writing upslog() type messages to the syslog */
+/* enable writing upslog_with_errno() and upslogx() type messages to
+   the syslog */
 void syslogbit_set(void);
 
 /* Return the default path for the directory containing configuration files */
@@ -80,19 +87,20 @@ const char * dflt_statepath(void);
 /* Return the alternate path for pid files */
 const char * altpidpath(void);
 
-void upslog(int priority, const char *fmt, ...)
+void upslog_with_errno(int priority, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
 void upslogx(int priority, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
-void upsdebug(int level, const char *fmt, ...)
+void upsdebug_with_errno(int level, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
 void upsdebugx(int level, const char *fmt, ...)
 	__attribute__ ((__format__ (__printf__, 2, 3)));
-
-void fatal(const char *fmt, ...)
-	__attribute__ ((__format__ (__printf__, 1, 2)));
+void upsdebug_hex(int level, const char *msg, const unsigned char *buf, int len);
+		
+void fatal_with_errno(const char *fmt, ...)
+	__attribute__ ((__format__ (__printf__, 1, 2))) __attribute((noreturn));
 void fatalx(const char *fmt, ...)
-	__attribute__ ((__format__ (__printf__, 1, 2)));
+	__attribute__ ((__format__ (__printf__, 1, 2))) __attribute((noreturn));
 
 extern int nut_debug_level;
 
@@ -101,7 +109,7 @@ void *xcalloc(size_t number, size_t size);
 void *xrealloc(void *ptr, size_t size);
 char *xstrdup(const char *string);
 
-void rtrim(char *in, char sep);
+char *rtrim(char *in, const char sep);
 
 /* Buffer sizes used for various functions */
 #define SMALLBUF	512
