@@ -117,15 +117,15 @@ static void check_ups(upstype *ups)
 	if ((!ups) || (!ups->fn))
 		return;
 
+	/* see if we need to (re)connect to the socket */
+	if (ups->sock_fd == -1)
+		ups->sock_fd = sstate_connect(ups);
+
 	/* throw some warnings if it's not feeding us data any more */
 	if (sstate_dead(ups, maxage))
 		ups_data_stale(ups);
 	else
 		ups_data_ok(ups);
-
-	/* see if we need to (re)connect to the socket */
-	if (ups->sock_fd == -1)
-		ups->sock_fd = sstate_connect(ups);
 }
 
 /* create a listening socket for tcp connections */
@@ -135,13 +135,13 @@ static void setuptcp(void)
 	int	res, one = 1;
 
 	if ((listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
-		fatal("socket");
+		fatal_with_errno("socket");
 
 	res = setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (void *) &one, 
 		sizeof(one));
 
 	if (res != 0)
-		fatal("setsockopt(SO_REUSEADDR)");
+		fatal_with_errno("setsockopt(SO_REUSEADDR)");
 
 	memset(&server, '\0', sizeof(server));
 	server.sin_addr = listenaddr;
@@ -149,16 +149,16 @@ static void setuptcp(void)
 	server.sin_port = htons(net_port);
 
 	if (bind(listenfd, (struct sockaddr *) &server, sizeof(server)) == -1)
-		fatal("Can't bind TCP port number %d", net_port);
+		fatal_with_errno("Can't bind TCP port number %d", net_port);
 
 	if ((res = fcntl(listenfd, F_GETFL, 0)) == -1)
-		fatal("fcntl(get)");
+		fatal_with_errno("fcntl(get)");
 
 	if (fcntl(listenfd, F_SETFL, res | O_NDELAY) == -1)
-		fatal("fcntl(set)");
+		fatal_with_errno("fcntl(set)");
 
 	if (listen(listenfd, 16))
-		fatal("listen");
+		fatal_with_errno("listen");
 
 	return;
 }
@@ -262,7 +262,7 @@ int sendback(ctype *client, const char *fmt, ...)
 		res = write(client->fd, ans, len);
 
 	if (len != res) {
-		upslog(LOG_NOTICE, "write() failed for %s", client->addr);
+		upslog_with_errno(LOG_NOTICE, "write() failed for %s", client->addr);
 		client->delete = 1;
 		return 0;	/* failed */
 	}
@@ -703,88 +703,6 @@ static void setupsignals(void)
 	sigaction(SIGHUP, &sa, NULL);
 }
 
-/* try not to exit before the DUMPDONE hits, so clients work on the first try */
-static void initial_dump_wait(void)
-{
-	int	maxfd, ret, numups, numdone;
-	fd_set	rfds;
-	time_t	start, now;
-	struct	timeval	tv;
-	upstype	*utmp, *unext;
-
-	printf("Synchronizing...");
-	fflush(stdout);
-
-	time(&start);
-	time(&now);
-
-	while (difftime(now, start) < INITIAL_WAIT_MAX) {
-
-		/* check this now in case the user is trying to ^C us */
-		if (exit_flag) {
-			upsd_cleanup();
-			exit(EXIT_FAILURE);
-		}
-
-		maxfd = 0;
-		numups = 0;
-		numdone = 0;
-
-		tv.tv_sec = 1;
-		tv.tv_usec = 0;
-
-		FD_ZERO(&rfds);
-
-		for (utmp = firstups; utmp != NULL; utmp = utmp->next) {
-			if (utmp->sock_fd != -1) {
-				FD_SET(utmp->sock_fd, &rfds);
-
-				if (utmp->sock_fd > maxfd)
-					maxfd = utmp->sock_fd;
-
-				numups++;
-			}
-		}
-
-		ret = select(maxfd + 1, &rfds, NULL, NULL, &tv);
-
-		if (ret < 1) {
-			printf(".");
-			fflush(stdout);
-
-			time(&now);
-			continue;
-		}
-
-		utmp = firstups;
-		while (utmp) {
-			unext = utmp->next;
-
-			if (utmp->sock_fd != -1) {
-				if (FD_ISSET(utmp->sock_fd, &rfds))
-					sstate_sock_read(utmp);
-
-				if (utmp->dumpdone == 1)
-					numdone++;
-			}
-
-			utmp = unext;
-		}
-
-		/* if they're all done, then exit early */
-		if (numdone >= numups) {
-			printf("done\n");
-			fflush(stdout);
-			return;
-		}
-
-		time(&now);
-	}
-
-	printf(" giving up\n");
-	fflush(stdout);
-}
-
 void check_perms(const char *fn)
 {
 	int	ret;
@@ -793,7 +711,7 @@ void check_perms(const char *fn)
 	ret = stat(fn, &st);
 
 	if (ret != 0)
-		fatal("stat %s", fn);
+		fatal_with_errno("stat %s", fn);
 
 	/* include the x bit here in case we check a directory */
 	if (st.st_mode & (S_IROTH | S_IXOTH))
@@ -835,7 +753,7 @@ int main(int argc, char **argv)
 				break;
 			case 'i':
 				if (!inet_aton(optarg, &listenaddr))
-					fatal("Invalid IP address");
+					fatal_with_errno("Invalid IP address");
 				break;
 			case 'r':
 				chroot_path = optarg;
@@ -895,8 +813,7 @@ int main(int argc, char **argv)
 		statepath = xstrdup(nut_statepath_env);
 
 	/* do this here, since getpwnam() might not work in the chroot */
-	if ((new_uid = get_user_pwent(user)) == NULL)
-		fatal("getpwnam(%s)", user);
+	new_uid = get_user_pwent(user);
 
 	if (chroot_path)
 		chroot_start(chroot_path);
@@ -904,7 +821,7 @@ int main(int argc, char **argv)
 	become_user(new_uid);
 
 	if (chdir(statepath))
-		fatal("Can't chdir to %s", statepath);
+		fatal_with_errno("Can't chdir to %s", statepath);
 
 	/* check statepath perms */
 	check_perms(statepath);
@@ -920,12 +837,6 @@ int main(int argc, char **argv)
 	/* try to bring in the var/cmd descriptions */
 	desc_load();
 
-	/* try to get a full dump for each UPS before exiting */
-	initial_dump_wait();
-
-	/* this is after the uid change to detect permission problems */
-	check_every_ups();
-
 	if (do_background) {
 		background();
 
